/* K6517Bv5.c  */
/* Add loop of measurement of currents for calculation of means and stddev
 */
/*****************************************************************************
 * Refer to the language interface documentation for details on
 * which header and .obj files to include in your project.
 *****************************************************************************/
#define FAKE 0
#define VERSION 5.1
#include <ansi_c.h>

#include "ni4882.h"
#include <stdio.h>		/*  for printf */
#include <stdlib.h>		/* for strtod */
#include <time.h>
#include "configuration.h"	/* defines struct for holding configuration data */
// will now be define as a -D parameter in Makefile
#include <windows.h>
/*
#ifndef FAKE
#include <windows.h>
#endif
#ifdef FAKEONWINDOWS
#include <windows.h>
#include <gpib.h>
#endif
*/
#ifdef FAKE
/* non default .............. */
       unsigned long ibsta;
       unsigned long iberr;
       int           cmdNumber;
       double        cmdValue;
/* end non default .............. */
#endif

#define DEBUG 0

/* signatures for functions... should be done in an include file ... */
void GpibError(const char * msg); /* Error function declaration              */
double calcMean( double * arrayOfValues, int lengthOfArray );
double calcStdDev( double * arrayOfValues, int lengthOfArray ); 
struct configuration readaConfiguration(char *filename, struct configuration IVconfiguration);

#define NLOOP 100
#define NAVE  100		
/* NAVE is number to calculate current mean and stddev at V-Source set point */

int Device = 0;                   /* Device unit descriptor                  */
int BoardIndex = 0;               /* Interface Index (GPIB0=0,GPIB1=1,etc.)  */
/* int BoardIndex = 1;  */             /* for PD lab setup */
unsigned long lresult;	
int incode=0;					  /* for terminal io control */


int main (int argc, char *argv[])		  {
	
#ifdef FAKE
   extern    int     cmdNumber;
   extern    double  cmdValue;
#endif
   int debugLevel=0, printLevel=0;
   int i,j, kseries;
   int msleep1=100;		/* sleep time for set and read cmd in msec */
   int msleep2=20;		/* sleep time for multi-reads for averaging in msec */
   double Voltages[NLOOP], Currents[NLOOP], CurrentsStd[NLOOP];
   double deltaVoltage,setVoltage;
   double currentMean, currentStdDev;
   double currentsForAve[NAVE];
   int   PrimaryAddress = 22;      /* Primary address of the device           */
   int   SecondaryAddress = 0;    /* Secondary address of the device         */
   char  Buffer[101];             /* Read buffer                             */
   char  CmdBuffer[101];
   int lenCmd;
   int smode=0;
   int nave=10;
   char *filename="configuration.dat";
   struct configuration IVconfiguration;
   // for getting date and time
   char tbuffer[256];
   time_t curtime;
   struct tm *loctime;
   char tstr[80];
   int tlen;
   // 
   // Read configuration information from an external file
   IVconfiguration=readaConfiguration(filename, IVconfiguration);
   
   /* FILE *fd = fopen("c:\\cygwin\\home\\may\\data.txt","w");		 */
   FILE *fd1 = fopen("data.txt","w");		/* output file for raw data in gnuplot format */
   FILE *fd2 = fopen("debug.txt","w");
   FILE *fd3 = fopen("metadata.txt","w"); /* output file to store metadata for 
						this run in gnuplot include file format */
   if (FAKE) {
	printf("Using FAKE Hardware API...\n");
	fprintf(fd1,"# Using FAKE Hardware API...\n");
   }

   fprintf(fd3,"PROG = \"%s\"\n",argv[0]);
   /* Get the current time.  */
   curtime = time (NULL);
   /* Convert it to local time representation.  */
   loctime = localtime (&curtime);
   /* Print out the date and time in the standard format.  */
   fputs (asctime (loctime), stdout);
   strcpy(tstr,asctime (loctime));
   tlen=strlen(tstr);
   if (tstr[tlen-1] == '\n') tstr[tlen-1]=0;	// strip \n from end of string
   fprintf(fd3,"TAKEN = \"%s\"\n", tstr );	// 
   fprintf(fd3,"NAVE = %d\n", nave);

   printf("Version VERSION .. getting configuration data from file: %s\n ",filename);
   printf("Description: %s\n,", IVconfiguration.Description);
   msleep1 = IVconfiguration.msleep1;
   msleep2 = IVconfiguration.msleep2;
   debugLevel=IVconfiguration.debugLevel;
   printLevel=IVconfiguration.printLevel;
   double VSource[NLOOP];
   if ( IVconfiguration.numLoops > 10 ) { printf(" Problem with numLoops... exit!\n"); return 1; }
   printf("Number of reads to use in averaging: %d\n", IVconfiguration.numForAve);
   nave = IVconfiguration.numForAve;
   if ( nave > NAVE ) { printf("nave is larger than array size...exit!\n"); return 1;}

/*****************************************************************************
 * Initialization - Done only once at the beginning of your application.
 *****************************************************************************/

   Device = ibdev(                /* Create a unit descriptor handle         */
         BoardIndex,              /* Board Index (GPIB0 = 0, GPIB1 = 1, ...) */
         PrimaryAddress,          /* Device primary address                  */
         SecondaryAddress,        /* Device secondary address                */
         T10s,                    /* Timeout setting (T10s = 10 seconds)     */
         1,                       /* Assert EOI line at end of write         */
         0);                      /* EOS termination mode                    */
   if (ibsta & ERR) {           /* Check for GPIB Error                    */
      GpibError("ibdev Error"); 
   }

   ibclr(Device);                 /* Clear the device                        */
   if (ibsta & ERR) {
      GpibError("ibclr Error");
   }

/*****************************************************************************
 * Main Application Body - Write the majority of your GPIB code here.
 *****************************************************************************/

   ibwrt(Device, "*IDN?", 5);     /* Send the identification query command   */
   if (FAKE) {cmdNumber=10; cmdValue=0.0;}
   if (ibsta & ERR) {
      GpibError("ibwrt Error");
   }

   lresult=ibrd(Device, Buffer, 100);     /* Read up to 100 bytes from the device    */
   if(printLevel >= 2) printf("ibrd lresult=%ld\n",lresult);
   if (ibsta & ERR) {
      GpibError("ibrd Error");	
   }
   
   if ( debugLevel >= 1 ) {
	   fprintf(fd2,"length of Buffer is %d of 100.\n",strlen(Buffer));
	   fprintf(fd2,"for *IDN? ibrd Buffer: %s\n",Buffer);
	   Buffer[55]='\000';
	   fprintf(fd2,"After truncation Buffer:%s\n",Buffer);
   }
   
   if (printLevel >= 2) printf("For *IDN* read back, iberr = %ld \n",iberr);
   Buffer[55]='\000';
   printf("For *IDN* read back, Buffer: %s\n", Buffer);        /* Print the device identification         */
   
   printf("Paused waiting at 2 for user input[enter 0 to continue]: "); scanf("%d", &incode);
   printf("Sleeping 1 sec Before the *RST reset command.\n");
   Sleep(1000);
   ibwrt(Device, "*RST", 4);
   
   printf("Sleeping 1 sec Before the 3 setup commands.\n");
   Sleep(1000);
   ibwrt(Device, ":SYST:ZCH 0", 11);		/* turn off zero check mode */
   /* ibwrt(Device,":CURR:DC:RANG 0.001",20);*/ /* set range .may be un needed */
   ibwrt(Device, ":OUTP:STAT 1", 12);		/* turn on V-Source */
   printf("Sleeping 1 sec Before reading V-Source\n");
   Sleep(1000);
   ibwrt(Device, ":SOUR:VOLT:LEV:IMM:AMPL?", 24);
   if (FAKE) {cmdNumber=11; cmdValue=0.0;}
   Sleep(1000);
   lresult=ibrd(Device, Buffer, 100);
   if (printLevel >= 2) printf("ibrd lresult=%ld\n", lresult);
   if (ibsta & ERR )  { GpibError("ibrd Error"); }
   if (printLevel >= 2) printf("size=%d Buffer: %s\n",strlen(Buffer),Buffer);
   if ( debugLevel >= 1 )fprintf(fd2, "From V-Source read Buffer:\n %s\n",Buffer);
   printf("Read Source Voltage: %s\n",Buffer); 
   fprintf(fd1,"# mode = %d msleep1 = %d (msec) msleep2 = %d PM IV data\n",smode,msleep1,msleep2);
   fprintf(fd1, "# index, V-Source, Current, Resistance, Power,\n");	   
   for ( kseries = 0; kseries<IVconfiguration.numLoops; kseries++) {
   for (i=0;i<IVconfiguration.numberOfIntervals[kseries];i++)		  {
	printf("\nTake reading.%d\n",i);
	Sleep(msleep1);
	// setup and send a value for V-Source
        deltaVoltage = (IVconfiguration.stopValues[kseries]-IVconfiguration.startValues[kseries])/IVconfiguration.numberOfIntervals[kseries];
        setVoltage = IVconfiguration.startValues[kseries] + i*deltaVoltage;
        Voltages[i] = setVoltage;
		sprintf(CmdBuffer,":SOUR:VOLT:LEV:IMM:AMPL %5.3f", Voltages[i]);
	lenCmd = strlen(CmdBuffer);
	if (printLevel >= 2) printf("sending Cmd: %s length of Cmd = %d\n",CmdBuffer, lenCmd);
	ibwrt(Device, CmdBuffer,lenCmd);
	if (FAKE) {cmdNumber=12; cmdValue=Voltages[i];}
	Sleep(msleep1);
	ibwrt(Device, ":SOUR:VOLT:LEV:IMM:AMPL?", 24);
	Sleep(msleep1);
	if (printLevel >= 2) printf("length of Buffer is %ld\n", strlen(Buffer));
	for (j=0; j<101; j++) Buffer[j]='\0';
	if (printLevel >= 2) printf("length of Buffer is %ld\n", strlen(Buffer));
	lresult=ibrd(Device, Buffer, 100);
	if (printLevel >= 2) printf("ibrd lresult=%ld\n", lresult);
	if (printLevel >= 2) printf("length of Buffer is %ld\n", strlen(Buffer));
	if (printLevel >= 2) printf("V-Source readback Buffer=%s\n",Buffer);
	Buffer[16]='\0';
	if (printLevel >= 2) printf("length of Buffer is %ld\n", strlen(Buffer));
	if (debugLevel >= 2) printf("Read back V-Source: %s\n",Buffer);
	if(debugLevel >= 2) fprintf(fd2, "From V-Source read Buffer:\n %s\n",Buffer); 
	Voltages[i]=strtod(Buffer,NULL);
	if (printLevel >= 1) printf("at i=%d deltaVoltage=%G setVoltage=%G readback=%G\n", i, deltaVoltage,setVoltage,Voltages[i]);
	
	/* setup a loop for multiple reads of currents to get averages and stddev */ 
    for (j=0; j<nave; j++) currentsForAve[j]=0.0;
	for ( j=0; j<nave; j++ ) {
		Sleep(msleep2);
		ibwrt(Device, ":FUNC 'CURR:DC'; :READ?", 23);
		if (FAKE) {cmdNumber=13; cmdValue=Voltages[i];}
		Sleep(msleep2);
		lresult=ibrd(Device, Buffer, 100);
		if(printLevel >= 3) printf("ibrd lresult=%ld Buffer length=%d\n", lresult, strlen(Buffer));
		if (ibsta & ERR )  { GpibError("ibrd Error"); }
		Buffer[23]='\000';
		if (printLevel >= 3) printf("At j=%d Read current, Buffer:%s\n",j,Buffer);
		if (debugLevel >= 3) fprintf(fd2, "At j=%d Read current, Buffer:%s\n",j,Buffer);
		if (ibsta & ERR )  { GpibError("ibrd Error"); }
		Buffer[16]='\000';
		if (printLevel >= 3) printf("After truncating Buffer:%s\n",Buffer);
		if(debugLevel >= 3) fprintf(fd2, "Trial=%d Buffer=%s\n",i,Buffer);
		currentsForAve[j]=strtod(Buffer,NULL);
		if (printLevel >= 2) printf("at j=%d currentForAve=%G Buffer=%s\n",j,currentsForAve[j],Buffer);
		if (printLevel >= 1) printf("currentsForAve[%d]=%G\n",j,currentsForAve[j]);
		if (debugLevel >= 3) fprintf(fd2, "currentsForAve[%d]=%G\n",j,currentsForAve[j]);
	} // end of for (j=0; j<nave...
	currentMean = calcMean( currentsForAve, nave );
	Currents[i]=currentMean;
	currentStdDev = calcStdDev( currentsForAve, nave );
	CurrentsStd[i]=currentStdDev;
	if (printLevel >= 0) printf("At i=%d V-Source = %G Iave=%G STD=%G\n",i,Voltages[i],currentMean,currentStdDev);
   } // end of for (i=0; i<IVconfiguration.numberOfIntervals[kseries];i++)	/* end of a DAQ loop */

   /* write out a table of values on screen and file system */

   printf("\nindex\tnave\tVoltage    Current\t\t   StdDev\t\tResistance\t Power\n");
   fprintf(fd1, "#index\tnave\tVoltage    Current\t   StdDev\tResistance\t  Power\n");
   for (i=0;i<IVconfiguration.numberOfIntervals[kseries];i++) {
	   printf("%4d %4d %12.5G\t%12.5G\t%12.5G\t%12.5G\t%12.5G\n",i,nave, 
			Voltages[i],Currents[i],CurrentsStd[i],
			Voltages[i]/Currents[i], Voltages[i]*Currents[i]);
   	   fprintf(fd1,"%4d %4d %12.5G %12.5G %12.5G %12.5G\t%12.5G\n",i,nave,
			Voltages[i],Currents[i],CurrentsStd[i],
			Voltages[i]/Currents[i], Voltages[i]*Currents[i]);
   }
    printf("index\tnave\tVoltage    Current\t\t   StdDev\t\tResistance\t Power\n");
   fprintf(fd1, "#index\tnave\tVoltage    Current\t   StdDev\tResistance\t  Power\n");
   
 } /* end of a series of measurements */
	
/*****************************************************************************
 * Uninitialization - Done only once at the end of your application.
 *****************************************************************************/
   Sleep(100);
   printf("Reset Keithley 5617B with RST command.\n");
   ibwrt(Device, "*RST", 4);
   Sleep(100);
   ibonl(Device, 0);              /* Take the device offline                 */
   if (ibsta & ERR) {
      GpibError("ibonl Error");	
   }
   fclose(fd1);
   if (debugLevel >= 1) fclose(fd2);
   fclose(fd3);
   printf("%s","Reached end of main...waiting on user input[0]:");
   scanf("%d",&incode);
   Sleep(100);
}


/*****************************************************************************
 *                      Function GPIBERROR
 * This function will notify you that a NI-488 function failed by
 * printing an error message.  The status variable IBSTA will also be
 * printed in hexadecimal along with the mnemonic meaning of the bit
 * position. The status variable IBERR will be printed in decimal
 * along with the mnemonic meaning of the decimal value.  The status
 * variable IBCNT will be printed in decimal.
 *
 * The NI-488 function IBONL is called to disable the hardware and
 * software.
 *
 * The EXIT function will terminate this program.
 *****************************************************************************/
void GpibError(const char *msg) {

    printf("%s\n", msg);

    printf("ibsta = 0x%lx  <", ibsta);
    if (ibsta & ERR )  printf(" ERR");
    if (ibsta & TIMO)  printf(" TIMO");
    if (ibsta & END )  printf(" END");
    if (ibsta & SRQI)  printf(" SRQI");
    if (ibsta & RQS )  printf(" RQS");
    if (ibsta & CMPL)  printf(" CMPL");
    if (ibsta & LOK )  printf(" LOK");
    if (ibsta & REM )  printf(" REM");
    if (ibsta & CIC )  printf(" CIC");
    if (ibsta & ATN )  printf(" ATN");
    if (ibsta & TACS)  printf(" TACS");
    if (ibsta & LACS)  printf(" LACS");
    if (ibsta & DTAS)  printf(" DTAS");
    if (ibsta & DCAS)  printf(" DCAS");
    printf (" >\n");

    printf ("iberr = %ld", iberr);
    if (iberr == EDVR) printf(" EDVR <Driver error>\n");
    if (iberr == ECIC) printf(" ECIC <Not Controller-In-Charge>\n");
    if (iberr == ENOL) printf(" ENOL <No Listener>\n");
    if (iberr == EADR) printf(" EADR <Address error>\n");
    if (iberr == EARG) printf(" EARG <Invalid argument>\n");
    if (iberr == ESAC) printf(" ESAC <Not System Controller>\n");
    if (iberr == EABO) printf(" EABO <Operation aborted>\n");
    if (iberr == ENEB) printf(" ENEB <No GPIB board>\n");
    if (iberr == EOIP) printf(" EOIP <Async I/O in progress>\n");
    if (iberr == ECAP) printf(" ECAP <No capability>\n");
    if (iberr == EFSO) printf(" EFSO <File system error>\n");
    if (iberr == EBUS) printf(" EBUS <Command error>\n");
    if (iberr == ESTB) printf(" ESTB <Status byte lost>\n");
    if (iberr == ESRQ) printf(" ESRQ <SRQ stuck on>\n");
    if (iberr == ETAB) printf(" ETAB <Table Overflow>\n");
    if (iberr == ELCK) printf(" ELCK <Lock error>\n");
    if (iberr == EARM) printf(" EARM <Ibnotify rearm error>\n");
    if (iberr == EHDL) printf(" EHDL <Invalid Handle>\n");
    if (iberr == EWIP) printf(" EWIP <Wait already in progress>\n");
    if (iberr == ERST) printf(" ERST <Notification cancelled due to reset>\n");
    if (iberr == EPWR) printf(" EPWR <Power error>\n");
  
    printf("iberr = %lu\n", iberr);
    printf("\n");

    /* Call ibonl to take the device and interface offline */
    ibonl(Device, 0);
	printf("Reached end of error handler.");
	
	printf("%s \n","Reached end of error handler...waiting on 0");
    scanf("%d",&incode);
    exit(1);

}
